// Box.hpp
//
// Copyright 2012-2013 Roan Trail, Inc.
//
// This file is part of Tovero.
//
// Tovero is free software; you can redistribute it and/or modify it
// under the terms of the GNU Lesser General Public License
// version 2.1 as published by the Free Software Foundation.
//
// Tovero is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
// Lesser General Public License for more details.  You should have
// received a copy of the GNU Lesser General Public License along with
// Tovero. If not, see <http://www.gnu.org/licenses/>.

// Right parallelpiped, specialized Polyhedron (8 vertex)

#ifndef TOVERO_MATH_BOX_HPP_
#define TOVERO_MATH_BOX_HPP_

#include <tovero/math/geometry/Polyhedron_base.hpp>
#include <tovero/math/geometry/Point.hpp>
#include <tovero/math/geometry/Unitless.hpp>
#include <tovero/math/geometry/Vector.hpp>
#include <string>

namespace Roan_trail
{
  namespace Tovero_math
  {
    class Geometric_tolerances;
    class Polyhedron;

    class Box : public Polyhedron_base
    {
    public:
      // constructor/copy
      Box(const std::string& name = "");
      Box(const Point& vertex,
          const Vector& width,
          const Vector& height,
          const Vector& depth,
          const std::string& name = "");
      virtual Solid* clone_solid() const { return new Box(*this); }
      //   Note: let compiler generate default copy constructor and operator=
      // accessors/mutators
      const Point& vertex() const { return m_vertex; }
      void set_vertex(const Point& vertex) { m_vertex = vertex; }
      const Vector& width() const { return m_width; }
      void set_width(const Vector& width) { m_width = width; }
      const Vector& depth() const { return m_depth; }
      void set_depth(const Vector& depth) { m_depth = depth; }
      const Vector& height() const { return m_height; }
      void set_height(const Vector& height) { m_height = height; }
      // predicates
      virtual bool is_valid(const Geometric_tolerances& tolerances,
                            Roan_trail::Tovero_support::Error_param& return_error) const;
      // visitor
      virtual Solid_visitor::Visit_result accept(Solid_visitor& visitor) const { return visitor.visit(*this); }
      // other
      virtual std::string solid_class() const { return "Box"; }
      virtual Polyhedron& generalize() const;
      virtual Solid& specialize(const Geometric_tolerances& tolerances) const;
    protected:
      // destructor
      virtual ~Box() {}
    private:
      // solid parameters
      Point m_vertex; // lower left corner
      Vector m_width;
      Vector m_height;
      Vector m_depth;
    };
  }
}

#endif // TOVERO_MATH_BOX_HPP_
