# test_axis_aligned_box.rb
#
# Copyright 2012-2014 Roan Trail, Inc.
#
# This file is part of Tovero.
#
# Tovero is free software; you can redistribute it and/or modify it
# under the terms of the GNU Lesser General Public License
# version 2.1 as published by the Free Software Foundation.
#
# Tovero is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.  You should have
# received a copy of the GNU Lesser General Public License along with
# Tovero. If not, see <http://www.gnu.org/licenses/>.

# Ruby unit test (minitest) for the Axis_aligned_box class

require 'minitest/autorun'
require 'test/unit'
require 'libtovero_support_rb_1'
require 'libtovero_math_rb_1'

include Libtovero_support_rb_1
include Libtovero_math_rb_1

class TestAxisAlignedBox < Test::Unit::TestCase
  def setup
    @Tolerances = GeometricTolerances.new
  end

  #
  # Solid validity checks
  #

  def test_default
    box = AxisAlignedBox.new
    error = ErrorParam.new
    #
    assert box.is_valid?(@Tolerances, error)
  end

  def test_invalid_min_max
    box = AxisAlignedBox.new
    box.name = "box"
    p = Point.new(1.0, 1.0, 1.0, Distance::meter)
    box.minimum = p
    box.maximum = p
    error = ErrorParam.new
    #
    refute box.is_valid?(@Tolerances, error)
    assert (error.base.error_class == 'Math_error') and (error.base.code.equal? MathError::Validation)
  end

  #
  # Miscellaneous checks
  #

  def test_properties
    box = AxisAlignedBox.new
    #
    assert_respond_to box, :minimum
    assert_respond_to box, :minimum=
    assert_respond_to box, :maximum
    assert_respond_to box, :maximum=
  end

end
